#include "matrix_tests.h"
#include <math.h>
#include "rgb_matrix.h"
#include "matrix_gfx.h"
#include "blocking_delay.h"

//This method allows you to draw a heart that is 7*6 big. x, y is the top left corner 
//*00*00*
//0000000
//0000000
//*00000*
//**000**
//***0***
//
void draw_heart(short x, short y, unsigned short color) {
    matrix_drawLine(x, y, x + 1, y, color);
    matrix_drawLine(x + 3, y, x + 4, y, color);
    matrix_drawLine(x - 1, y + 1, x + 5, y + 1, color);
    matrix_drawLine(x - 1, y + 2, x + 5, y + 2, color);
    matrix_drawLine(x, y + 3, x + 4, y + 3, color);
    matrix_drawLine(x + 1, y + 4, x + 3, y + 4, color);
    matrix_drawLine(x + 2, y + 5, x + 2, y + 5, color);
}

//draw a happy face, x, y is the center of the circle.
void draw_happy_face(short x, short y, unsigned short face_color, unsigned short color) {
    matrix_drawCircle(x, y, 3, face_color);
    //eyes
    matrix_drawPixel(x - 1, y - 1, color);
    matrix_drawPixel(x + 1, y - 1, color);
    //mouth
    matrix_drawLine(x - 1, y + 2, x + 1, y + 2, color);
    matrix_drawPixel(x - 2, y + 1, color);
    matrix_drawPixel(x + 2, y + 1, color);
}

//draw a neutral face, x, y is the center of the circle.
void draw_neutral_face(short x, short y, unsigned short face_color, unsigned short color) {
    matrix_drawCircle(x, y, 3, face_color);
    //eyes
    matrix_drawPixel(x - 1, y - 1, color);
    matrix_drawPixel(x + 1, y - 1, color);
    //mouth
    matrix_drawLine(x - 2, y + 1, x + 2, y + 1, color);
}

//draw a angry face, x, y is the center of the circle.
void draw_angry_face(short x, short y, unsigned short face_color, unsigned short color) {
    matrix_drawCircle(x, y, 3, face_color);
    //eyes
    matrix_drawLine(x - 2, y - 1, x - 1, y, color);
    matrix_drawLine(x + 2, y - 1, x + 1, y, color);
    //mouth
    matrix_drawLine(x - 1, y + 2, x + 1, y + 2, color);
}

//draw a sad face, x, y is the center of the circle.
void draw_sad_face(short x, short y, unsigned short face_color, unsigned short color) {
    matrix_drawCircle(x, y, 3, face_color);
    //eyes
    matrix_drawPixel(x - 1, y - 1, color);
    matrix_drawPixel(x + 1, y - 1, color);
    //mouth
    matrix_drawPixel(x, y + 1, color);
    matrix_drawPixel(x - 1, y + 2, color);
    matrix_drawPixel(x + 1, y + 2, color);
}

//draw a pet
void draw_pet(short x, short y, unsigned short color) {
    //head (default 5,5)
    matrix_drawRect(x, y, 9, 6, color);
    //body
    matrix_drawRect(x+2, y+5, 5, 4, color); 
}

//move the left hand
void move_left_hand(short move_x, short move_y, short fix_x, short fix_y, unsigned short color){
     //hand(default 4, 12)
    //fix x and fix y should be the same as the draw pet pixel
    matrix_drawLine(move_x, move_y, fix_x+2, fix_y+7, color); 
}

//move the right hand
void move_right_hand(short move_x, short move_y, short fix_x, short fix_y, unsigned short color){
     //hand(default 14, 12)
    //fix x and fix y should be the same as the draw pet pixel
    matrix_drawLine(fix_x+7, fix_y+7, move_x, move_y, color);     
}

//move feet
void move_feet(short x, short y, unsigned short color){
    //feet(default 5, 14)
    matrix_drawRect(x, y, 3, 2, color);
    matrix_drawRect(x+6, y, 3, 2, color);   
}

//draw a neutral face on the pet
void draw_pet_neutral_face(short x, short y, unsigned short eye_color, unsigned short mouth_color){
    //eyes default x = 5, y = 5 (same as the draw_pet)
    matrix_drawPixel(x+2, y+2,  eye_color);
    matrix_drawPixel(x+6, y+2,  eye_color);
    //mouth
    matrix_drawLine(x+2, y+4, x+6, y+4, mouth_color); 
}

//draw a pet happy face
void draw_pet_happy_face(short x, short y, unsigned short eye_color, unsigned short mouth_color){
    //eyes default x = 5, y = 5 (same as the draw_pet)
    matrix_drawPixel(x+2, y+1,  eye_color);
    matrix_drawPixel(x+6, y+1,  eye_color); 
 
    //mouth
    matrix_drawLine(x+2, y+4, x+6, y+4, mouth_color);
    matrix_drawPixel(x+1, y+3,  mouth_color);
    matrix_drawPixel(x+7, y+3,  mouth_color);  
}

//happy face animation part 1
void happy_expression1(){
    
    matrix_drawLine(2, 13, 2, 15, matrix_color333(7, 7, 0));
    matrix_drawLine(16, 13, 16, 15, matrix_color333(7, 7, 0));
}
//happy face animation part 2
void happy_expression2(){
    matrix_drawLine(2, 10, 2, 11, matrix_color333(7, 7, 0));
    matrix_drawLine(16, 10, 16, 11, matrix_color333(7, 7, 0));    
}
//happy face animation part 3
void happy_expression3(){
    matrix_drawLine(2, 7, 2, 9, matrix_color333(7, 7, 0));
    matrix_drawLine(16, 7, 16, 9, matrix_color333(7, 7, 0));    
}
//happy face animation part 4
void happy_expression4(){
    matrix_drawLine(1, 6, 3, 8, matrix_color333(7, 7, 0));
    matrix_drawLine(1, 7, 3, 7, matrix_color333(7, 7, 0));   
    matrix_drawLine(1, 8, 3, 6, matrix_color333(7, 7, 0));
    
    matrix_drawLine(15, 6, 17, 8, matrix_color333(7, 7, 0));
    matrix_drawLine(15, 7, 17, 7, matrix_color333(7, 7, 0));   
    matrix_drawLine(15, 8, 17, 6, matrix_color333(7, 7, 0));
}
//happy face animation part 5
void happy_expression5(){
    matrix_drawLine(0, 5, 4, 9, matrix_color333(7, 7, 0));
    matrix_drawLine(4, 5, 0, 9, matrix_color333(7, 7, 0));   
    matrix_drawLine(2, 5, 2, 9, matrix_color333(7, 7, 0));
    matrix_drawLine(0, 7, 4, 7, matrix_color333(7, 7, 0)); 
    
    matrix_drawLine(14, 5, 18, 9, matrix_color333(7, 7, 0));
    matrix_drawLine(18, 5, 14, 9, matrix_color333(7, 7, 0));   
    matrix_drawLine(16, 5, 16, 9, matrix_color333(7, 7, 0));
    matrix_drawLine(14, 7, 18, 7, matrix_color333(7, 7, 0)); 
}
//draw a sad face on the pet
void draw_pet_sad_face(short x, short y, unsigned short eye_color, unsigned short mouth_color){
    //eyes default x = 5, y = 5 (same as the draw_pet)
    matrix_drawPixel(x+2, y+1,  eye_color);
    matrix_drawPixel(x+6, y+1,  eye_color); 
 
    //mouth
    matrix_drawLine(x+3, y+3, x+5, y+3, mouth_color);
    matrix_drawPixel(x+2, y+4,  mouth_color);
    matrix_drawPixel(x+6, y+4,  mouth_color);  
}
//sad face animation part 1
void draw_pet_sad_expression1(){
    matrix_drawLine(7, 7, 6, 8, matrix_color333(0, 7, 7));
    matrix_drawLine(11, 7, 12, 8, matrix_color333(0, 7, 7));
}

//sad face animation part 2
void draw_pet_sad_expression2(){
    matrix_drawLine(4, 10, 3, 11, matrix_color333(0, 7, 7));
    matrix_drawLine(14, 10, 15, 11, matrix_color333(0, 7, 7));
}

//sad face animation part 3
void draw_pet_sad_expression3(){
    matrix_drawLine(1, 13, 0, 14, matrix_color333(0, 7, 7));
    matrix_drawLine(17, 13, 18, 14, matrix_color333(0, 7, 7));
}

//draw an angry face on the pet
void draw_pet_angry_face(short x, short y, unsigned short eye_color, unsigned short mouth_color){
    //eyes default x = 5, y = 5 (same as the draw_pet)
    matrix_drawLine(x+2, y+1,x+3, y+2, eye_color);
    matrix_drawLine(x+5, y+2, x+6, y+1, eye_color);
 
    //mouth
    matrix_drawLine(x+2, y+4, x+6, y+4, mouth_color);  
}
//angry face animation
void draw_pet_angry_expression(){
    matrix_drawLine(0, 6, 4, 6, matrix_color333(7, 0, 0));
    matrix_drawLine(0, 8, 4, 8, matrix_color333(7, 0, 0));
    matrix_drawLine(1, 5 , 1, 9, matrix_color333(7, 0, 0));
    matrix_drawLine(3, 5, 3, 9, matrix_color333(7, 0, 0));
    
    matrix_drawLine(14, 6, 18, 6, matrix_color333(7, 0, 0));
    matrix_drawLine(14, 8, 18, 8, matrix_color333(7, 0, 0));
    matrix_drawLine(15, 5 , 15, 9, matrix_color333(7, 0, 0));
    matrix_drawLine(17, 5, 17, 9, matrix_color333(7, 0, 0));

}
//draw a tired face on the face 
void draw_pet_tired_face(short x, short y, unsigned short eye_color, unsigned short mouth_color){
    //eyes default x = 5, y = 5 (same as the draw_pet)
    matrix_drawLine(x+1, y+2, x+2, y+2, eye_color);
    matrix_drawLine(x+6, y+2, x+7, y+2, eye_color);
    //mouth
    matrix_drawLine(x+3, y+4, x+5, y+4, mouth_color); 
}

//tired face animation
void draw_pet_tired_expression(){
    //left z
    matrix_drawLine(0, 5, 4, 5, matrix_color333(7, 7, 0));
    matrix_drawLine(4, 5, 0, 9, matrix_color333(7, 7, 0));
    matrix_drawLine(0, 9, 4, 9, matrix_color333(7, 7, 0));
    
    //right z
    matrix_drawLine(14, 5, 18, 5, matrix_color333(7, 7, 0));
    matrix_drawLine(18, 5, 14, 9, matrix_color333(7, 7, 0));
    matrix_drawLine(14, 9, 18, 9, matrix_color333(7, 7, 0));

}